#!/usr/bin/env python

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- http://www.linuxnorth.org/raspi-sump/license.html

from raspisump import checkpid


def main():
    '''run checkpid.py module to restart Raspi-Sump if the rsump.py process is
    stopped or has spawned multiple processes.'''
    process = 'rsump.py'
    checkpid.check_pid(process)

if __name__ == "__main__":
    main()



'''Monitor the health of a process and restart it if it has stopped
or if it has spawned multiple processes.'''

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- http://www.linuxnorth.org/raspi-sump/license.html

import subprocess
from raspisump import log

def check_pid(process):
    '''Check status of process.'''
    cmdp1 = 'ps aux'
    cmdp2 = 'grep -v grep'
    cmdp3 = 'grep -v sudo'
    cmdp4 = 'grep -c ' + process
    cmdp1list = cmdp1.split(' ')
    cmdp2list = cmdp2.split(' ')
    cmdp3list = cmdp3.split(' ')
    cmdp4list = cmdp4.split(' ')
    part1 = subprocess.Popen(cmdp1list, stdout=subprocess.PIPE)
    part2 = subprocess.Popen(cmdp2list, stdin=part1.stdout,
                             stdout=subprocess.PIPE
                             )
    part1.stdout.close()
    part3 = subprocess.Popen(cmdp3list, stdin=part2.stdout,
                             stdout=subprocess.PIPE
                             )
    part2.stdout.close()
    part4 = subprocess.Popen(cmdp4list, stdin=part3.stdout,
                             stdout=subprocess.PIPE
                             )
    part3.stdout.close()
    number_of_processes = int(part4.communicate()[0])
    if number_of_processes == 0:
        log.log_restarts('Process stopped, restarting')
        restart(process)
    elif number_of_processes == 1:
        exit(0)
    else:
        log.log_restarts('Multiple processes...killing and restarting')
        kill_start(process)


def restart(process):
    '''Restart process'''
    restart_cmd = process + ' &'
    restart_now = restart_cmd.split(' ')
    subprocess.Popen(restart_now)
    exit(0)


def kill_start(process):
    '''Kill all instances of process and initiate restart.'''
    kill_cmd = 'killall 09 ' + process
    kill_it = kill_cmd.split(' ')
    subprocess.call(kill_it)
    restart(process)



'''Create charts for viewing on Raspberry Pi Web Server.'''

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- http://www.linuxnorth.org/raspi-sump/license.htmlimport os

import os
import subprocess
import time
from raspisump import todaychart


def create_folders(year, month, homedir):
    '''Check if folders exist in charts folder and create them if they don't'''
    if not os.path.isdir('{}charts/{}/'.format(homedir, year)):
        _year = 'mkdir {}charts/{}'.format(homedir, year)
        create_year = _year.split(' ')
        subprocess.call(create_year)

    if not os.path.isdir('{}charts/{}/{}/'.format(homedir, year, month)):
        _month = 'mkdir {}charts/{}/{}'.format(homedir, year, month)
        create_month = _month.split(' ')
        subprocess.call(create_month)


def create_chart(homedir):
    '''Create a chart of sump pit activity and save to web folder'''
    csv_file = '{}csv/waterlevel-{}.csv'.format(
        homedir, time.strftime('%Y%m%d')
        )
    filename = '{}charts/today.png'.format(homedir)
    bytes2str = todaychart.bytesdate2str('%H:%M:%S')
    todaychart.graph(csv_file, filename, bytes2str)


def copy_chart(year, month, today, homedir):
    '''Copy today.png to year/month/day folder for web viewing'''
    copy_cmd = 'cp {}charts/today.png {}charts/{}/{}/{}.png'.format(
        homedir, homedir, year, month, today
        )
    copy_file = copy_cmd.split(' ')
    subprocess.call(copy_file)



#!/usr/bin/env python

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- http://www.linuxnorth.org/raspi-sump/license.html

import time

try:
    import ConfigParser as configparser # Python2
except ImportError:
    import configparser # Python3
from raspisump import reading

config = configparser.RawConfigParser()

config.read('/home/pi/raspi-sump/raspisump.conf')
reading_interval = config.getint('pit', 'reading_interval')

if reading_interval == 0:
    reading.water_depth()
else:
    while True:
        reading.water_depth()
        time.sleep(reading_interval)



'''Graph sump pit activity.'''

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- http://www.linuxnorth.org/raspi-sump/license.html

import time
import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from matplotlib import rcParams
rcParams.update({'figure.autolayout': True})
try:
    import ConfigParser as configparser # Python2
except ImportError:
    import configparser # Python3

config = configparser.RawConfigParser()
config.read('/home/pi/raspi-sump/raspisump.conf')
configs = {'unit': config.get('pit', 'unit')}

MPL_VERSION = int(mpl.__version__.split(".")[0]) # Matplotlib major version

if MPL_VERSION > 1:
    rcParams['date.autoformatter.hour'] = '%H:%M:%S' # Matplotlib 2.0 changed time formatting
    
def bytesdate2str(fmt, encoding='utf-8'):
    '''Convert strpdate2num from bytes to string as required in Python3.

    This is a workaround as described in the following tread;
    https://github.com/matplotlib/matplotlib/issues/4126/

    Credit to github user cimarronm for this workaround.
    '''

    strconverter = mdates.strpdate2num(fmt)

    def bytesconverter(b):
        s = b.decode(encoding)
        return strconverter(s)
    return bytesconverter


def graph(csv_file, filename, bytes2str):
    '''Create a line graph from a two column csv file.'''

    unit = configs['unit']
    date, value = np.loadtxt(csv_file, delimiter=',', unpack=True,
                             converters={0: bytes2str}
                             )
    fig = plt.figure(figsize=(10, 3.5))
    
    # axisbg is deprecated in matplotlib 2.x. Maintain 1.x compatibility
    if MPL_VERSION > 1:
        fig.add_subplot(111, facecolor='white', frameon=False)
    else:
        fig.add_subplot(111,axisbg='white', frameon=False)
    
    rcParams.update({'font.size': 9})
    plt.plot_date(x=date, y=value, ls='solid', linewidth=2, color='#FB921D',
                  fmt=':'
                  )
    title = "Sump Pit Water Level {}".format(time.strftime('%Y-%m-%d %H:%M'))
    title_set = plt.title(title)
    title_set.set_y(1.09)
    plt.subplots_adjust(top=0.86)

    if unit == 'imperial':
        plt.ylabel('inches')
    if unit == 'metric':
        plt.ylabel('centimeters')

    plt.xlabel('Time of Day')
    plt.xticks(rotation=30)
    plt.grid(True, color='#ECE5DE', linestyle='solid')
    plt.tick_params(axis='x', bottom='off', top='off')
    plt.tick_params(axis='y', left='off', right='off')
    plt.savefig(filename, dpi=72)



'''Log waterlevel readings, restarts and alerts.'''

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# http://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- http://www.linuxnorth.org/raspi-sump/license.html

import time


def log_reading(water_depth):
    '''Log time and water depth reading.'''
    time_of_reading = time.strftime("%H:%M:%S,")
    filename = "/home/pi/raspi-sump/csv/waterlevel-{}.csv".format(
        time.strftime("%Y%m%d")
    )
    csv_file = open(filename, 'a')
    csv_file.write(time_of_reading),
    csv_file.write(str(water_depth)),
    csv_file.write("\n")
    csv_file.close()


def log_restarts(reason):
    '''Log all process restarts'''
    logfile = open("/home/pi/raspi-sump/logs/process_log", 'a')
    logfile.write(time.strftime("%Y-%m-%d %H:%M:%S,")),
    logfile.write(reason),
    logfile.write("\n")
    logfile.close()


def log_alerts(notification):
    '''Log all email sms alerts'''
    alert_log = open("/home/pi/raspi-sump/logs/alert_log", 'a')
    alert_log.write(time.strftime("%Y-%m-%d %H:%M:%S,")),
    alert_log.write(notification),
    alert_log.write("\n")
    alert_log.close()


def log_errors(notification):
    '''Log all errors'''
    error_log = open("/home/pi/raspi-sump/logs/error_log", 'a')
    error_log.write(time.strftime("%Y-%m-%d %H:%M:%S,")),
    error_log.write(notification),
    error_log.write("\n")
    error_log.close()

